import { WidgetModel } from '@jupyter-widgets/base';
import ndarray = require('ndarray');
export { ISerializers } from '@jupyter-widgets/base';
/**
 * An interface for a data source.
 */
export interface IDataSource {
    getNDArray(key?: string): ndarray.NdArray | null;
}
/**
 * An interface for a data widget that can be written back to.
 *
 * Widgets that sync the data back and forth between the kernel and
 * frontend should implement this, while widgets that generate the
 * data from a process without a reverse can choose not to.
 */
export interface IDataWriteBack {
    setNDArray(array: ndarray.NdArray | null, key?: string, options?: any): void;
    canWriteBack(key?: string): boolean;
}
/**
 * A widget that might expose IDataSource and/or IDataSink.
 */
export declare type DataWidget = WidgetModel & IDataSource & Partial<IDataWriteBack>;
/**
 * Whether an object implements the data source interface.
 */
export declare function isDataSource(candidate: any): candidate is IDataSource;
/**
 * Whether an object implements the data source interface.
 */
export declare function isDataWriteBack(candidate: any): candidate is IDataWriteBack;
/**
 * Gets the array of any array source.
 */
export declare function getArray(source: ndarray.NdArray | IDataSource | null, key?: string): ndarray.NdArray | null;
/**
 * Sets the union attribute of a widget to an array, in place.
 *
 * If current value of the attribute is a data sink, (i.e. implements the
 * setNDArray method), it will set its data. Otherwise it will set
 * the attribute value to the array.
 *
 * Note: If you never want to overwrite the data of a widget reference,
 * instead just call `widget.set(key, array)` directly.
 */
export declare function setArray(widget: WidgetModel, attrName: string, array: ndarray.NdArray | null, options?: any): void;
//# sourceMappingURL=common.d.ts.map