"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.setArray = exports.getArray = exports.isDataWriteBack = exports.isDataSource = void 0;
/**
 * Whether an object implements the data source interface.
 */
function isDataSource(candidate) {
    return candidate && typeof candidate.getNDArray === 'function';
}
exports.isDataSource = isDataSource;
/**
 * Whether an object implements the data source interface.
 */
function isDataWriteBack(candidate) {
    return candidate && typeof candidate.setNDArray === 'function';
}
exports.isDataWriteBack = isDataWriteBack;
/**
 * Gets the array of any array source.
 */
function getArray(source, key) {
    if (isDataSource(source)) {
        return source.getNDArray(key);
    }
    return source;
}
exports.getArray = getArray;
/**
 * Sets the union attribute of a widget to an array, in place.
 *
 * If current value of the attribute is a data sink, (i.e. implements the
 * setNDArray method), it will set its data. Otherwise it will set
 * the attribute value to the array.
 *
 * Note: If you never want to overwrite the data of a widget reference,
 * instead just call `widget.set(key, array)` directly.
 */
function setArray(widget, attrName, array, options) {
    const current = widget.get(attrName);
    if (isDataWriteBack(current) && current.canWriteBack()) {
        current.setNDArray(array, undefined, options);
    }
    else {
        widget.set(attrName, array, options);
    }
}
exports.setArray = setArray;
//# sourceMappingURL=common.js.map