import { IWidgetManager, WidgetModel } from '@jupyter-widgets/base';
import { IReceivedSerializedArray, ISendSerializedArray, typedArrayToJSON, simpleToJSON, TypedArray, ISimpleObject } from './ndarray';
import { DataWidget } from './common';
import ndarray = require('ndarray');
/**
 * Union type declaration of an NDArrayModel and a raw ndarray.
 */
export declare type DataUnion = DataWidget | ndarray.NdArray;
/**
 * Deserializes union JSON to an ndarray or a NDArrayModel, as appropriate.
 */
export declare function JSONToUnion(obj: IReceivedSerializedArray | string | null, manager?: IWidgetManager): Promise<ndarray.NdArray | DataWidget | null>;
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
export declare function JSONToUnionArray(obj: IReceivedSerializedArray | string | null, manager?: IWidgetManager): Promise<ndarray.NdArray | null>;
/**
 * Serializes a union to JSON.
 */
export declare function unionToJSON(obj: DataUnion | null, widget?: WidgetModel): ISendSerializedArray | string | null;
/**
 * Sets up backbone events for listening to union changes.
 *
 * The callback will be called when:
 *  - The model is a widget, and its data changes
 *
 * Specify `allChanges` as truthy to also cover these cases:
 *  - The union changes from a widget to an array or vice-versa
 *  - The union is an array and its content changes
 *
 * To stop listening, call the return value.
 */
export declare function listenToUnion(model: WidgetModel, unionName: string, callback: (model: WidgetModel, options: any) => any, allChanges?: boolean): () => void;
export declare const data_union_array_serialization: {
    deserialize: typeof JSONToUnionArray;
    serialize: typeof unionToJSON;
};
export declare const data_union_serialization: {
    deserialize: typeof JSONToUnion;
    serialize: typeof unionToJSON;
};
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
export declare function JSONToUnionTypedArray(obj: IReceivedSerializedArray | string | null, manager?: IWidgetManager): Promise<TypedArray | null>;
export declare const data_union_typedarray_serialization: {
    deserialize: typeof JSONToUnionTypedArray;
    serialize: typeof typedArrayToJSON;
};
/**
 * Deserializes union JSON to an ndarray, regardless of whether it is a widget reference or direct data.
 */
export declare function JSONToSimpleUnion(obj: IReceivedSerializedArray | string | null, manager?: IWidgetManager): Promise<ISimpleObject | null>;
export declare const data_union_simple_serialization: {
    deserialize: typeof JSONToSimpleUnion;
    serialize: typeof simpleToJSON;
};
//# sourceMappingURL=union.d.ts.map